@@ -5,7 +5,7 @@
  *  open source, and has the attribution requirements (GPL Section 7) at
  *  http://statnet.org/attribution
  *
- *  Copyright 2003-2013 Statnet Commons
+ *  Copyright 2003-2014 Statnet Commons
  */
 #include "MCMCDyn.h"
 
@@ -104,6 +104,8 @@ void MCMCDyn_wrapper(// Starting network.
   Model *F_m, *D_m, *M_m;
   MHproposal F_MH, D_MH;
 
+  if(*lasttoggle == 0) lasttoggle = NULL;
+
   Vertex *difftime, *difftail, *diffhead;
   int *diffto;
 
@@ -150,6 +152,8 @@ void MCMCDyn_wrapper(// Starting network.
   if(*status == MCMCDyn_OK && *maxedges>0){
     newnetworktails[0]=newnetworkheads[0]=EdgeTree2EdgeList(newnetworktails+1,newnetworkheads+1,nw,*maxedges-1);
     *time = nw->duration_info.time;
+
+    if(nw->duration_info.lasttoggle)
     memcpy(lasttoggle, nw->duration_info.lasttoggle, sizeof(int)*DYADCOUNT(*n_nodes, *bipartite, *dflag));
   }
 
@@ -160,7 +164,6 @@ void MCMCDyn_wrapper(// Starting network.
   }
 
   MCMCDyn_finish_common(nw, F_m, D_m, M_m, &F_MH, &D_MH);
-
 }
 
 /*********************
@@ -503,4 +506,3 @@ MCMCDynStatus MCMCDyn1Step(// Observed and discordant network.
     *nextdiffedge=nde;
   return MCMCDyn_OK;
 }
-
